@@ -77,7 +77,7 @@ MessagesModel::MessagesModel(const Tp::AccountPtr &account, QObject *parent) :
     //Load configuration for number of message to show
     KConfig config(QLatin1String("ktelepathyrc"));
     KConfigGroup tabConfig = config.group("Behavior");
-    d->logManager->setScrollbackLength(tabConfig.readEntry<int>("scrollbackLength", 10));
+    d->logManager->setScrollbackLength(tabConfig.readEntry<int>("scrollbackLength", 20));
 }
 
 QHash<int, QByteArray> MessagesModel::roleNames() const
@@ -178,18 +178,49 @@ void MessagesModel::setAccount(const Tp::AccountPtr &account)
     d->account = account;
 }
 
+void MessagesModel::setContactData(const QString &contactId, const QString &contactAlias)
+{
+    d->logManager->setAccountAndContact(d->account, contactId, contactAlias);
+
+    //Load messages unless they have already been loaded
+    if (!d->logsLoaded) {
+        qDebug() << "Fetching scrollback";
+        d->logManager->fetchScrollback();
+    }
+}
+
 void MessagesModel::onHistoryFetched(const QList<KTp::Message> &messages)
 {
-    if (!messages.isEmpty()) {
+    QList<KTp::Message> messagesToAdd;
+
+    // Make sure we're not adding duplicated messages to the model
+    if (!d->messages.isEmpty()) {
+        int i = 0;
+        for (i = 0; i < messages.size(); i++) {
+            if (messages.at(i) == d->messages.at(0).message) {
+                break;
+            }
+        }
+        messagesToAdd = messages.mid(0, i);
+    } else {
+        messagesToAdd = messages;
+    }
+
+    if (!messagesToAdd.isEmpty()) {
         //Add all messages before the ones already present in the channel
-        beginInsertRows(QModelIndex(), 0, messages.count() - 1);
-        for(int i=messages.size()-1;i>=0;i--) {
-            d->messages.prepend(messages[i]);
+        beginInsertRows(QModelIndex(), 0, messagesToAdd.count() - 1);
+        for (int i = messagesToAdd.size() - 1; i >= 0; i--) {
+            d->messages.prepend(messagesToAdd[i]);
         }
         endInsertRows();
     }
     d->logsLoaded = true;
 
+    // Emit changed for the first message after the prepended
+    // logs, to make sure the bubble shape is updated
+    // through PreviousMessageTypeRole
+    QModelIndex index = createIndex(messagesToAdd.count(), 0);
+    Q_EMIT dataChanged(index, index);
     Q_EMIT lastMessageChanged();
 }
 
@@ -374,6 +405,11 @@ void MessagesModel::sendNewMessage(const QString &message)
     if (message.isEmpty()) {
         qCWarning(KTP_DECLARATIVE) << "Attempting to send empty string, this is not supported";
     } else {
+        if (!d->textChannel) {
+            qWarning(KTP_DECLARATIVE) << "Attempting to send a message without a channel, returning";
+            return;
+        }
+
         Tp::PendingOperation *op;
         QString modifiedMessage = message;
         if (d->textChannel->supportsMessageType(Tp::ChannelTextMessageTypeAction)
@@ -473,3 +509,19 @@ QDateTime MessagesModel::lastMessageDateTime() const
 
     return data(index, MessagesModel::TimeRole).toDateTime();
 }
+
+void MessagesModel::fetchMoreHistory()
+{
+    if (d->messages.isEmpty() || !d->logsLoaded) {
+        return;
+    }
+
+    d->logsLoaded = false;
+
+    const KTp::Message message = d->messages.at(0).message;
+
+    const QString token = message.token().isEmpty() ? message.time().toString(Qt::ISODate) + message.mainMessagePart()
+                                                    : message.token();
+    d->logManager->setScrollbackLength(10);
+    d->logManager->fetchHistory(rowCount() + 10, token);
+}
